import { RegistrationResult } from './../../models/RegistrationResult';
import { HTTP_HELPER } from './../../providers/http.helper.provider';
import { Captcha } from './../../models/Captcha';
import { EventService } from 'src/app/services/event.service';
import { environment } from './../../../environments/environment';
import { FinalizeRegistrationRequest } from './../../models/FinalizeRegistrationRequest';
import { Sponsorship } from '../../models/Sponsorship';
import { Speaker } from '../../models/Speaker';
import { SessionTrack } from '../../models/SessionTrack';
import { Session } from '../../models/Session';
import { Pass } from '../../models/Pass';
import { HttpHelper } from '../../helpers/HttpHelper';
import { Injectable, Inject } from '@angular/core';
import { Event } from '../../models/Event';
import { Observable } from 'rxjs';
import { RegistrationData } from '../../models/RegistrationData';
import * as CustomRegistrationFieldModel from '../../models/CustomRegistrationField';
import { shareReplay } from 'rxjs/operators';

@Injectable()
export class EventRestService implements EventService {
    private static readonly eventsEndpoint: string = 'events';

    constructor(@Inject(HTTP_HELPER) private http: HttpHelper) {
    }

    public getPublishedEvents(): Observable<Event[]> {
        return this.http.get<Event[]>(`${environment.apiEndpoint}${EventRestService.eventsEndpoint}/published`);
    }

    public event$: Map<string, Observable<Event>> = new Map();
    public resetEvent(): void {
        this.event$ = new Map();
    }

    public getEvent(readableEventId: string): Observable<Event> {
        if (!this.event$.has(readableEventId)) {
            this.event$.set(readableEventId, this.http.get<Event>(
                `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}`).pipe(shareReplay(1)));
        }
        return this.event$.get(readableEventId);
    }

    public passes$: Observable<Pass[]>;
    public resetPasses(): void {
        this.passes$ = null;
    }

    public getPasses(readableEventId: string): Observable<Pass[]> {
        if (!this.passes$) {
            this.passes$ = this.http.get<Pass[]>(
                `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/passes`
            ).pipe(shareReplay(1));
        }
        return this.passes$;
    }

    public session$: Observable<Session[]>;
    public resetSessions(): void {
        this.session$ = null;
    }

    public getSessions(readableEventId: string): Observable<Session[]> {
        if (!this.session$) {
            this.session$ = this.http.get<Session[]>(
                `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/sessions`
            ).pipe(shareReplay(1));
        }
        return this.session$;
    }

    public getSessionTracks(readableEventId: string): Observable<SessionTrack[]> {
        return this.http.get<SessionTrack[]>(
            `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/tracks`
        );
    }

    public getSpeakers(readableEventId: string): Observable<Speaker[]> {
        return this.http.get<Speaker[]>(
            `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/speakers`
        );
    }

    public getSponsors(readableEventId: string): Observable<Sponsorship[]> {
        return this.http.get<Sponsorship[]>(
            `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/sponsorships`
        );
    }

    getCaptcha(readableEventId: string): Observable<Captcha> {
        return this.http.get<Captcha>(
            `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/captcha`
        );
    }

    public getCustomRegistrationFields(readableEventId: string): Observable<CustomRegistrationFieldModel.CustomRegistrationField[]> {
        const url = `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/custom-registration-fields`;
        return this.http.get<CustomRegistrationFieldModel.CustomRegistrationField[]>(url);
    }

    public count$: Observable<number>;
    public resetCount(): void {
        this.count$ = null;
    }

    public getEventRegistrationCount(readableEventId: string): Observable<number> {
        if (!this.count$) {
            this.count$ = this.http.get<number>(
                `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/registrations/count`
            ).pipe(shareReplay(1));
        }
        return this.count$;
    }

    public registerToEvent(readableEventId: string, registrationData: RegistrationData): Observable<RegistrationResult> {
        const url = `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/registrations`;
        return this.http.post<RegistrationResult>(url, registrationData, true);
    }

    public finalizeRegistration(readableEventId: string, requestData: FinalizeRegistrationRequest): Observable<RegistrationResult> {
        const url = `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/registrations/finalize`;
        return this.http.post<RegistrationResult>(url, requestData, true);
    }

    public registerToSession(readableEventId: string, sessionId: string): Observable<boolean> {
        return this.http.post<boolean>(
            `${environment.apiEndpoint}${EventRestService.eventsEndpoint}/${readableEventId}/sessions/${sessionId}/registrations`, {}, true
        );
    }
}
